{ Contains the code for all file input and output. All I/O should be done using
  these procedures.  }
unit iFileio;

interface

uses sysutils, Dialogs, stypes, isomain;

type  TAction = (flRead, flWrite);

procedure ReadSpecFile(SpecFilename:string);
procedure OpenDataFile(filename:string; var names:namelist; var units:namelist);
function ReadDataFile(var time:double; var data:datavalues;
                       var X,F,delF,R:stvec{;var R:coefarray}):Boolean;
procedure CloseDataFile;
procedure SaveOutput(name:string);
procedure ParseString(str:string; sep:string; var arr:namelist);

var
 specfile, datafile:textfile;

implementation

uses idisplay;

procedure ReadSpecFile(SpecFilename:string);
var
 i:integer;
begin
 assignfile(specfile, SpecFilename);
 try
   reset(specfile);
   readln(specfile,Ncol); //# of columns in data file
   readln(specfile,qbase); //ratio of heavy to light isotope in standard 
   readln(specfile,nstat); //# of state variables
   for i:=1 to nstat do
    begin
       readln(specfile,{names[i],}colx[i],del[i]);
    end;
   readln(specfile,nflux); //# of fluxes
   for i:=1 to nflux do
    begin
       readln(specfile,Rfrom[i],Rto[i],colR[i],Rd[i]);
       del[nstat+i] := Rd[i];
    end;
   readln(specfile,nforce); //# of forcing fluxes
   for i:=1 to nforce do readln(specfile,Fto[i],colF[i],delFcol[i]);
 finally
   closefile(specfile);
 end;
end;

// Data File I/O

{ The data file is a space delimited ASCII file containing values for the
  state variables and processes at each time step of the model run. The first
  item on a line is the time followed by the state variables and then the
  process variables. The first two lines of the file are header lines. The first
  line contains the names of the variables and the second line contains the
  units.

  The data file must be opened and closed using OpenDataFile and
  CloseDataFile. Be sure to call CloseDataFile when you are done.}

// Opens the data file for reading
procedure OpenDataFile(filename:string; var names:namelist; var units:namelist);
var
  j:integer;
  tempstring, comma:string;
begin
 comma := ',';
 assignfile(datafile,filename);
 reset(datafile);
 readln(datafile, tempstring);
 Parsestring(tempstring, comma, names);
 readln(datafile,tempstring);
 ParseString(tempstring, comma, units);
 for j := ncol + 1 to ncol + nstat do
   begin
     names[j] := 'del ' + names[colX[j-ncol]];
     units[j] := 'del ' + units[colX[j-ncol]];
   end;
 for j := ncol + nstat + 1 to ncol + nstat + nflux do
   begin
     names[j] := 'del ' + names[colR[j-ncol-nstat]];
     units[j] := 'del ' + units[colR[j-ncol-nstat]];
   end;
end;  // opendatafile

procedure ParseString(str:string; sep:string; var arr:namelist);
var
 index, count:integer;
begin
 count := 1;
 index := pos(sep,str);
 while index > 0 do
  begin
    arr[count] := copy(str,1,index-1);
    delete(str,1,index);
    index := pos(sep,str);
    count := count + 1;
  end;
 arr[count] := str;
end;

// Reads one line of the data file
function ReadDataFile(var time:double; var data:datavalues;
                       var X,F,delF,R:stvec{;var R:coefarray}):Boolean;
var
   i:integer;
   tempstring, comma:string;
   dumdat:namelist;
 //  dumdat:mat;
begin
 comma := ',';
 if not eof(datafile) then
  begin
   readln(datafile, tempstring);
   ParseString(tempstring,comma,dumdat);
   for i := 1 to ncol do data[i] := strtofloat(dumdat[i]);
   time:=data[1];
   for i:= 1 to nstat do X[i]:=data[colX[i]];
   for i:= 1 to nflux do R[i{Rto[i],Rfrom[i]}]:=data[colR[i]];
   for i:= 1 to nforce do
      begin
         F[Fto[i]]:=data[colF[i]];
         delF[Fto[i]]:=data[delFcol[i]];
      end;
   ReadDataFile := True;
  end
 else
   ReadDataFile := False;
end;

// Closes the data file
procedure CloseDataFile;
begin
   closefile(datafile);
end;

procedure SaveOutput(name:string);
begin
 FmDisplayData.SgModelOutput1.Savetoxls(name);
end;

end.

